<?php

namespace Netzmacher\Browser\Utility\Update;

use Netzmacher\Browser\Utility\ObjectUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2017-2019 -  Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 *
 * @author  Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package TYPO3
 * @subpackage quick_shop
 * @version     9.4.1
 * @since       9.4.1
 */
class ScriptXMLUpdate
{

	/**
	 * @var string
	 */
	private $ll = 'LLL:EXT:browser/Resources/Private/Language/Update/scriptXMLUpdate.xml:';

	/**
	 * @var array
	 */
	protected $oldTags = [
			'"record_browser"',
			'FIELD:record_browser:',
			'"template.tutorial"',
			'"template.templateTutorial"',
	];

	/**
	 * @var array
	 */
	protected $newTags = [
			'"record_browser.mode"',
			'FIELD:record_browser.mode:',
			'"templateTutorial"',
			'"templateTutorialDeprecated"',
	];

	/**
	 * Updates deprecated XML tags within Browser plugins
	 *
	 * @return	string		HTML
	 * @version     9.4.1
	 * @since       9.4.1
	 */
	private function _XMLUpdate()
	{
		$content = NULL;

		foreach( array_keys( ( array ) $this->newTags ) as $key )
		{
			$content = ''
							. $content
							. '<li>' . PHP_EOL
							. $this->oldTags[ $key ] . ' is moved to ' . $this->newTags[ $key ] . ' in #' . $this->_updateTag( $key ) . ' Browser plugins'
							. '</li>' . PHP_EOL
			;
		}

		$content = ''
						. '<h3>' . PHP_EOL
						. 'Details'
						. '</h3>' . PHP_EOL
						. '<ul>' . PHP_EOL
						. $content
						. '</ul>' . PHP_EOL
		;
		return ''
						. $this->_prompt( 'promptTagUpdate', 'ok' )
						. $this->_form( 'formFALSuccessHeader', $content, 'formReloadButton' )
		;
	}

	/**
	 * Return the HTML form
	 * 
	 * @param string $legend:
	 * @param string $content:
	 * @param string $button:
	 * @param string $action:
	 *
	 * @return	string		HTML
	 * @version     9.4.1
	 * @since       9.4.1
	 */
	private function _form( $legend, $content, $button, $action = NULL )
	{
		if( $legend )
		{
			$legend = '<legend>' . $GLOBALS[ 'LANG' ]->sL( $this->ll . $legend ) . '</legend>';
		}
		return '
      <form action="" method="POST">
        ' . $legend . '
        <fieldset>
          ' . $this->_formInput( $action ) . '
          ' . $content . '
          ' . $this->_formButton( $button ) . '
        </fieldset>
			</form>
      <p>&nbsp;</p>
      '
		;
	}

	/**
	 * Return the HTML button
	 * 
	 * @param string $button:
	 *
	 * @return	string		HTML
	 * @version     9.4.1
	 * @since       9.4.1
	 */
	private function _formButton( $button )
	{
		return ''
						. '<input type="submit" class="button" value="' . $GLOBALS[ 'LANG' ]->sL( $this->ll . $button ) . '">' . PHP_EOL
		;
	}

	/**
	 * Return the input fields for the HTML form
	 * 
	 * @param string $action:
	 *
	 * @return	string		HTML
	 * @version     9.4.1
	 * @since       9.4.1
	 */
	private function _formInput( $action = NULL )
	{
		$moduleToken = \TYPO3\CMS\Core\Utility\GeneralUtility::_GP( 'moduleToken' );

		if( $action )
		{
			$action = '<input type="hidden" name="action" value="' . $action . '" />' . PHP_EOL;
		}

		return ''
						. '<input type="hidden" name="moduleToken" value="' . $moduleToken . '" />' . PHP_EOL
						. '<input type="hidden" name="M" value="tools_ExtensionmanagerExtensionmanager" />' . PHP_EOL
						. '<input type="hidden" name="tx_extensionmanager_tools_extensionmanagerextensionmanager[extensionKey]" value="browser" />' . PHP_EOL
						. '<input type="hidden" name="tx_extensionmanager_tools_extensionmanagerextensionmanager[action]" value="show" />' . PHP_EOL
						. '<input type="hidden" name="tx_extensionmanager_tools_extensionmanagerextensionmanager[controller]" value="UpdateScript" />' . PHP_EOL
						. $action
		;
	}

	/**
	 * Returns the number of the affected rows
	 * 
	 * @param integer $tagKey Kes of the current tag
	 * @return	string		HTML
	 * @version     9.4.1
	 * @since       9.4.1
	 */
	private function _getNumberOfAffectedPlugins( $tagKey = NULL )
	{
		$select = 'uid';
		$from = 'tt_content';
		$where = $this->_getNumberOfAffectedPluginsWhere( $tagKey );
//		$SELECTquery = ObjectUtility::getDatabaseConnection()->SELECTquery($select, $from, $where);
//		var_dump( __METHOD__, __LINE__, $SELECTquery );
		$res = ObjectUtility::getDatabaseConnection()->exec_SELECTquery( $select, $from, $where );
//		var_dump( __METHOD__, __LINE__, $res->num_rows );

		return $res->num_rows;
	}

	/**
	 * Returns the where statement
	 *
	 * @param integer $tagKey Kes of the current tag
	 * @return	string		HTML
	 * @version     9.4.1
	 * @since       9.4.1
	 */
	private function _getNumberOfAffectedPluginsWhere( $tagKey = NULL )
	{
		$whereDefault = 'list_type = \'browser_pi1\' AND CType = \'list\'';

		switch( TRUE )
		{
			case($tagKey != NULL):
				$whereOldTags = 'pi_flexform LIKE \'%' . $this->oldTags[ $tagKey ] . '%\'';
				break;
			case($tagKey == NULL):
			default:
				$whereOldTags = 'pi_flexform LIKE \'%' . implode( '%\' OR pi_flexform LIKE \'%', $this->oldTags ) . '%\'';
				break;
		}
		$where = $whereDefault . ' AND (' . $whereOldTags . ')';

		return $where;
	}

	/**
	 * Returns the menu
	 *
	 * @return	string		The menu
	 * @version     9.4.1
	 * @since       9.4.1
	 */
	private function _menu()
	{
		$numRows = $this->_getNumberOfAffectedPlugins();

		switch( TRUE )
		{
			case(!$numRows ):
				return $this->_form( 'formTagUpdateHeader', $this->_menuXMLTagEmpty(), 'formReloadButton' );
			case( $numRows ):
			default:
				return $this->_form( 'formTagUpdateHeader', $this->_menuXMLTagList( $numRows ), 'formTagUpdateButton', 'updateXMLTags' );
		}
	}

	/**
	 * List XML tags, which should updated
	 *
	 * @return	string		HTML
	 * @version     9.4.1
	 * @since       9.4.1
	 */
	private function _menuXMLTagEmpty()
	{
		return $this->_prompt( 'promptTagUpdateIsDone', 'ok' );
	}

	/**
	 * List XML tags, which should updated
	 *
	 * @param integer $numRows: number of affected rows
	 * @return	string		HTML
	 * @version     9.4.1
	 * @since       9.4.1
	 */
	private function _menuXMLTagList( $numRows )
	{

		switch( TRUE )
		{
			case(!$numRows):
				$input = ''
								. '<p>'
								. $this->_prompt( 'promptFALIsDone', 'ok' )
								. '</p>' . PHP_EOL
				;
				break;
			case( $numRows ):
			default:
				$input = ''
								. $this->_prompt( 'promptBackupDb', 'information' )
								. '<p>'
								. $GLOBALS[ 'LANG' ]->sL( $this->ll . 'promptAffectedPluginsPrompt' ) . $numRows
								. '</p>' . PHP_EOL
				;
				break;
		}

		return $input;
	}

	/**
	 * Returns the HTML prompt
	 * 
	 * @param string $prompt: the prompt
	 * @param string $severity: severity
	 *
	 * @return	string		HTML prompt
	 * @version     9.4.1
	 * @since       9.4.1
	 */
	private function _prompt( $prompt, $severity = 'information' )
	{
		// #i0262, 181025, dwildt, 4+
		$alertSeverity = $severity;
		switch( TRUE )
		{
			case( $alertSeverity == "information" ):
				$alertSeverity = "info";
				break;
			case( $alertSeverity == "ok" ):
				$alertSeverity = "success";
				break;
		}

		// #i0262, 181025, dwildt, ~
		return '
			<div class="alert alert-' . $alertSeverity . ' typo3-message message-' . $severity . '">
        <div class="message-header">' . $GLOBALS[ 'LANG' ]->sL( $this->ll . $prompt . 'Header' ) . '</div>
        <div class="message-body">
          ' . $GLOBALS[ 'LANG' ]->sL( $this->ll . $prompt . 'Prompt' ) . '
        </div>
      </div>' . PHP_EOL;
	}

	/**
	 * Returns the number of the affected rows
	 * 
	 * @param integer $tagKey: Key of the current tag
	 * @return	void
	 * @version     9.4.1
	 * @since       9.4.1
	 */
	private function _updateTag( $tagKey )
	{
		$oldTag = $this->oldTags[ $tagKey ];
		$newTag = $this->newTags[ $tagKey ];
		$numRows = $this->_getNumberOfAffectedPlugins( $tagKey );
		$query = ''
						. 'UPDATE tt_content '
						. 'SET pi_flexform = replace (pi_flexform, \'' . $oldTag . '\', \'' . $newTag . '\') '
						. 'WHERE list_type = \'browser_pi1\' '
						. 'AND CType = \'list\' '
						. 'AND pi_flexform like \'%' . $oldTag . '%\'';
		//var_dump( __METHOD__, __LINE__, $query );
		ObjectUtility::getDatabaseConnection()->sql_query( $query );

		return $numRows;
	}

	/**
	 * Main function, returning the HTML content of the update script
	 *
	 * @return	string		HTML
	 * @version     9.4.1
	 * @since       9.4.1
	 */
	public function main()
	{
		$action = \TYPO3\CMS\Core\Utility\GeneralUtility::_GP( 'action' );

		switch( TRUE )
		{
			case( $action == 'updateXMLTags'):
				return $this->_XMLUpdate();
			case(!empty( $action )):
				return NULL;
			default:
				return $this->_menu();
		}
	}

}