<?php

namespace Netzmacher\Browser\ViewHelpers\TxPowermail\v3011000\Getter;

use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface;
use TYPO3\CMS\Fluid\Core\ViewHelper\AbstractViewHelper;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2017-2019 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * Class BrowserViewHelper
 *
 * @package Netzmacher\Browser\ViewHelpers\TxPowermail\v3011000\Getter
 * @author Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @version 8.0.21
 * @since 8.0.7
 * @internal #i0119
 */
class BrowserViewHelper extends AbstractViewHelper
{

	/**
	 * @var TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface
	 * @inject
	 */
	protected $configurationManager;

	/**
	 * @var TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer
	 */
	protected $contentObjectRenderer;

	/**
	 * sqlRepository
	 *
	 * @var Netzmacher\Browser\Domain\Repository\SqlRepository
	 * @inject
	 */
	protected $sqlRepository;

	/**
	 * Current powermail field marker
	 *
	 * @var string
	 */
	private $_marker;

	/**
	 * Current local table
	 *
	 * @var string
	 */
	private $_tableLocal;

	/**
	 * Current foreign table
	 *
	 * @var string
	 */
	protected $_tableForeign;

	/**
	 * pid of the sysfolder or pid list of the sysfolders,
	 * which contains the records of the current table
	 *
	 * @var string
	 */
	private $_pidList;

	/**
	 * Initialize
	 *
	 * @access public
	 * @return void
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	public function initialize()
	{
		$this->contentObjectRenderer = $this->objectManager->get(
						'TYPO3\\CMS\\Frontend\\ContentObject\\ContentObjectRenderer'
		);
		$typoScriptSetup = $this->configurationManager->getConfiguration(
						ConfigurationManagerInterface::CONFIGURATION_TYPE_FULL_TYPOSCRIPT
		);
		// #i0291, 170117, dwildt, 1-/+
		//$this->settings = $typoScriptSetup[ 'plugin.' ][ 'tx_org.' ][ 'settings.' ];
		$this->settings = $typoScriptSetup[ 'plugin.' ][ 'tx_browser_pi6.' ][ 'settings.' ];
	}

	/**
	 * Arguments Initialization
	 *
	 * @access public
	 * @return void
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	public function initializeArguments()
	{
		$this->registerArgument( 'field', 'field', 'The field object ...', TRUE );
	}

	/**
	 * render( )  : Returns an array proper for the map property of the fluid alias.
	 *
	 * @access public
	 * @return array $map
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	public function render()
	{
		$this->_marker = $this->arguments[ 'field' ]->getMarker();

		list( $this->_tableLocal, $this->_tableForeign ) = explode( '__', $this->_marker );

		if( $this->_init() )
		{
			return FALSE;
		}

		$map = $this->_map();
		//var_dump(__METHOD__, __LINE__, $map);

		return $map;
	}

	/**
	 * _getPidList( ) :
	 *
	 * @access private
	 * @return string
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	protected function _getPidList()
	{
		return $this->_pidList;
	}

	/**
	 * _init( ) :
	 *
	 * @access private
	 * @return boolean
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _init()
	{
		$this->_setPidList();
		return FALSE;
	}

	/**
	 * _map( ) : Returns an array proper for the map property of the fluid alias.
	 *
	 * @access private
	 * @return array $rows
	 * @internal #i0178
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _map()
	{
		$rows = $this->_mapRows();
		//var_dump( __METHOD__, __LINE__, $rows );
		$rows = $this->_mapLabel( $rows );
		//var_dump( __METHOD__, __LINE__, $rows );
		return $rows;
	}

	/**
	 * _mapLabel( ) : Labels the rows array.
	 *                Is needed for the map property of the fluid alias.
	 *
	 * @access private
	 * @param array $rows
	 * @return array $map
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _mapLabel( $rows )
	{
		$map = array(
				'rows' => $rows
		);
		return $map;
	}

	/**
	 * _mapRows( ) :
	 *
	 * @access private
	 * @return array $rows
	 * @internal #i0178
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _mapRows()
	{
		$rows = $this->_mapRowsSql();
		return $rows;
	}

	/**
	 * _mapRowsSql( ) :
	 *
	 * @access private
	 * @return array $rows
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _mapRowsSql()
	{
		$query = $this->_mapRowsSqlQuery();
		$rows = $this->_mapRowsSqlExec( $query );
		return $rows;
	}

	/**
	 * _mapRowsSqlExec( ) :
	 *
	 * @access private
	 * @param string $query
	 * @return array $rows
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _mapRowsSqlExec( $query )
	{
		$rows = $this->sqlRepository->SELECT( $query );
//		var_dump(__METHOD__, __LINE__, $query, $rows);
		return $rows;
	}

	/**
	 * _mapRowsSqlQuery( ) :
	 *
	 * @access private
	 * @return string $query
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _mapRowsSqlQuery()
	{
		$config = BackendUtility::getTcaFieldConfiguration( $this->_tableLocal, $this->_tableForeign );
		//var_dump( __METHOD__, __LINE__, $config[ 'renderType' ] );
		switch( $config[ 'renderType' ] )
		{
			case('selectSingle'):
			case('selectMultipleSideBySide'):
				$query = $this->_mapRowsSqlQuerySelectSingle();
				break;
			case('selectTree'):
				$query = $this->_mapRowsSqlQuerySelectTree();
				break;
			default:
				$field = $this->_tableLocal . '.' .  $this->_tableForeign;
				$this->_mapRowsSqlQueryDie($field, __METHOD__, __LINE__);
				break;
		}

		return $query;
	}

	/**
	 * _mapRowsSqlQueryDie( ) :
	 *
	 * @access private
	 * @return string $query
	 * @version 8.0.22
	 * @since 8.0.21
	 */
	private function _mapRowsSqlQueryDie( $field, $method = __METHOD__, $line = __LINE__ )
	{
		$config = BackendUtility::getTcaFieldConfiguration( $this->_tableLocal, $this->_tableForeign );
		$header = 'ERROR: renderType is unknown';
		$text = ''
						. '<p>'
						. '  field is: "' . $field . '"'
						. '</p>'
						. '<p>'
						. '  renderType is: "' . $config[ 'renderType' ] . '"'
						. '</p>'
		;
		$this->_zzDieWiPrompt( $header, $text, $method, $line );
	}

	/**
	 * _mapRowsSqlQuerySelectSingle( ) :
	 *
	 * @access private
	 * @return string $query
	 * @internal #i0304
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	public function _mapRowsSqlQuerySelectSingle()
	{
		$query = '
      SELECT    title AS catParent,
                uid AS catParentUid
      FROM      ' . $this->_tableForeign . '
      WHERE     pid IN (' . $this->_getPidList() . ')
      ORDER BY  title;
    '
		;
		return $query;
	}

	/**
	 * _mapRowsSqlQuerySelectTree( ) :
	 *
	 * @access private
	 * @return string $query
	 * @internal #i0304
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _mapRowsSqlQuerySelectTree()
	{
		$query = '
      SELECT    t1.title AS catParent,
                t2.title AS catChild,
                t1.uid AS catParentUid,
                t2.uid AS catChildUid
      FROM      ' . $this->_tableForeign . ' AS t1
      LEFT JOIN ' . $this->_tableForeign . ' AS t2
                ON  t2.uid_parent = t1.uid
                AND t2.hidden     = 0
                AND t2.deleted    = 0
      WHERE         t1.uid_parent = 0
                AND t1.hidden     = 0
                AND t1.deleted    = 0
      AND       t1.pid IN (' . $this->_getPidList() . ')
      ORDER BY  t1.title, t2.title;
    '
		;
		return $query;
	}

	/**
	 * _setPidList( ) :
	 *
	 * @access private
	 * @return boolean
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _setPidList()
	{
		$data = array(
				'dummy' => 'dummy',
		);
		// #i0291, 170117, dwildt, 1-/+
		//$this->_tableForeign = 'tx_org_jobsection';
		$this->contentObjectRenderer->start( $data, $this->_tableForeign );
		// #i0291, 170117, dwildt, 4-
//		$value = $this->contentObjectRenderer->cObjGetSingle(
//						$this->settings[ 'constants.' ][ 'sysfolders.' ][ 'job' ]
//						, $this->settings[ 'constants.' ][ 'sysfolders.' ][ 'job.' ]
//		);
		// #i0291, 170117, dwildt, 4+
		$value = $this->contentObjectRenderer->cObjGetSingle(
						$this->settings[ 'pid.' ][ $this->_tableForeign ]
						, $this->settings[ 'pid.' ][ $this->_tableForeign . '.' ]
		);

		if( !empty( $value ) )
		{
			$this->_pidList = $value;
			return FALSE;
		}
		$this->_setPidListDie( __METHOD__, __LINE__ );
	}

	/**
	 * _setPidListDie( ) :
	 *
	 * @access private
	 * @return boolean
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _setPidListDie( $method, $line )
	{
		$header = 'ERROR: pid is emtpy';
		$text = ''
						. '<h2>'
						. '  Check your TypoScript'
						. '</h2>'
						. '<p>'
						. '  Please check your TypoScript setup:'
						. '</p>'
						. '<ul>'
						. '  <li>'
						. '    plugin.tx_browser_pi6.settings.pid.' . $this->_tableForeign
						. '  </li>'
						. '  <li>'
						. '    value must be the pid of the folder or the pid list of the folders,<br />'
						. '    which contain the records of the table "' . $this->_tableForeign . '"'
						. '  </li>'
						. '</ul>'
		;
		$this->_zzDieWiPrompt( $header, $text, $method, $line );
	}

	/**
	 * _zzDieWiPrompt( ) :
	 *
	 * @access private
	 * @return boolean
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _zzDieWiPrompt( $header, $text, $method, $line )
	{
		$prompt = '
      <h1 style="color:red;">
        ' . $header . '
      </h1>
      ' . $text . '
      <p>
        Sorry for the trouble. Browser - responsive TYPO3 without PHP.<br />
        Error occures here: ' . $method . ' at #' . $line . '
      </p>
      <p>
        If you need any help, please visit the
        <a href="http://typo3-browser-forum.de/" target="_blank" title="TYPO3 Browser Forums. 500 TYPO3-integrators are registered.">
          TYPO3 Browser Forum &raquo;</a>
      </p>
      ';
		die( $prompt );
	}

}
